@@ -18,6 +18,7 @@ from ..crawlers.Erome_Spider import EromeCrawler
 from ..crawlers.Gfycat_Spider import GfycatCrawler
 from ..crawlers.GoFile_Spider import GofileCrawler
 from ..crawlers.HGameCG_Spider import HGameCGCrawler
+from ..crawlers.ImgBox_Spider import ImgBoxCrawler
 from ..crawlers.Kemono_Spider import KemonoCrawler
 from ..crawlers.Pixeldrain_Spider import PixelDrainCrawler
 from ..crawlers.Postimg_Spider import PostImgCrawler
@@ -29,7 +30,7 @@ from ..crawlers.SocialMediaGirls_Spider import SocialMediaGirlsCrawler
 from ..crawlers.SimpCity_Spider import SimpCityCrawler
 from ..crawlers.XBunker_Spider import XBunkerCrawler
 from ..crawlers.XBunkr_Spider import XBunkrCrawler
-from ..base_functions.base_functions import log
+from ..base_functions.base_functions import log, write_last_post_file
 from ..base_functions.data_classes import CascadeItem, SkipData, AuthData
 from ..client.rate_limiting import AsyncRateLimiter
 
@@ -37,9 +38,10 @@ from ..client.rate_limiting import AsyncRateLimiter
 class ScrapeMapper:
     def __init__(self, *, client: Client, file_args: Dict, jdownloader_args: Dict, runtime_args: Dict,
                  jdownloader_auth: AuthData, simpcity_auth: AuthData, socialmediagirls_auth: AuthData,
-                 xbunker_auth: AuthData, skip_data: SkipData):
+                 xbunker_auth: AuthData, skip_data: SkipData, quiet: bool):
 
         self.include_id = runtime_args['include_id']
+        self.quiet = quiet
         self.jdownloader_enable = jdownloader_args['jdownloader_enable']
         self.jdownloader_device = jdownloader_args['jdownloader_device']
         self.separate_posts = runtime_args['separate_posts']
@@ -62,6 +64,7 @@ class ScrapeMapper:
         self.gfycat_crawler = None
         self.gofile_crawler = None
         self.hgamecg_crawler = None
+        self.imgbox_crawler = None
         self.kemono_crawler = None
         self.pixeldrain_crawler = None
         self.postimg_crawler = None
@@ -81,19 +84,27 @@ class ScrapeMapper:
         self.bunkr_limiter = AsyncRateLimiter(15)
         self.forum_limiter = asyncio.Semaphore(4)
         self.semaphore = asyncio.Semaphore(1)
+
+        self.crawlers = [self.anonfiles_crawler, self.bunkr_crawler, self.cyberdrop_crawler, self.coomer_crawler,
+                         self.cyberfile_crawler, self.erome_crawler, self.gfycat_crawler, self.gofile_crawler,
+                         self.hgamecg_crawler, self.imgbox_crawler, self.kemono_crawler, self.pixeldrain_crawler,
+                         self.postimg_crawler, self.redgifs_crawler, self.rule34_crawler, self.saint_crawler,
+                         self.sharex_crawler, self.socialmediagirls_crawler, self.simpcity_crawler,
+                         self.xbunker_crawler, self.xbunkr_crawler]
+
         self.mapping = {"anonfiles.com": self.Anonfiles, "bayfiles": self.Anonfiles, "xbunkr": self.XBunkr,
                         "bunkr": self.Bunkr, "coomer.party": self.Coomer, "cyberdrop": self.Cyberdrop,
                         "cyberfile.is": self.Cyberfile, "erome.com": self.Erome, "gfycat.com": self.Gfycat,
-                        "gofile.io": self.GoFile, "hgamecg.com": self.HGameCG, "img.kiwi": self.ShareX,
-                        "jpg.church": self.ShareX, "kemono.party": self.Kemono, "pixeldrain.com": self.Pixeldrain,
-                        "pixl.is": self.ShareX, "postimg": self.Postimg, "redgifs.com": self.Redgifs,
-                        "rule34.xxx": self.Rule34, "saint.to": self.Saint, "socialmediagirls": self.SocialMediaGirls,
-                        "simpcity": self.SimpCity, "xbunker": self.XBunker}
+                        "gofile.io": self.GoFile, "hgamecg.com": self.HGameCG, "imgbox.com": self.ImgBox,
+                        "img.kiwi": self.ShareX, "jpg.church": self.ShareX, "kemono.party": self.Kemono, 
+                        "pixeldrain.com": self.Pixeldrain, "pixl.li": self.ShareX, "postimg": self.Postimg,
+                        "redgifs.com": self.Redgifs, "rule34.xxx": self.Rule34, "saint.to": self.Saint, 
+                        "socialmediagirls": self.SocialMediaGirls, "simpcity": self.SimpCity, "xbunker": self.XBunker}
 
     async def Anonfiles(self, url: URL, title=None):
         anonfiles_session = Session(self.client)
         if not self.anonfiles_crawler:
-            self.anonfiles_crawler = AnonfilesCrawler(include_id=self.include_id)
+            self.anonfiles_crawler = AnonfilesCrawler(include_id=self.include_id, quiet=self.quiet)
         domain_obj = await self.anonfiles_crawler.fetch(anonfiles_session, url)
         if title:
             await domain_obj.append_title(title)
@@ -103,7 +114,7 @@ class ScrapeMapper:
     async def Bunkr(self, url: URL, title=None):
         bunkr_session = Session(self.client)
         if not self.bunkr_crawler:
-            self.bunkr_crawler = BunkrCrawler(include_id=self.include_id)
+            self.bunkr_crawler = BunkrCrawler(include_id=self.include_id, quiet=self.quiet)
         async with self.bunkr_limiter:
             domain_obj = await self.bunkr_crawler.fetch(bunkr_session, url)
         if title:
@@ -114,7 +125,7 @@ class ScrapeMapper:
     async def Cyberdrop(self, url: URL, title=None):
         cyberdrop_session = Session(self.client)
         if not self.cyberdrop_crawler:
-            self.cyberdrop_crawler = CyberdropCrawler(include_id=self.include_id)
+            self.cyberdrop_crawler = CyberdropCrawler(include_id=self.include_id, quiet=self.quiet)
         domain_obj = await self.cyberdrop_crawler.fetch(cyberdrop_session, url)
         if title:
             await domain_obj.append_title(title)
@@ -123,9 +134,11 @@ class ScrapeMapper:
 
     async def Coomer(self, url: URL, title=None):
         coomer_session = Session(self.client)
+        if self.output_last[0]:
+            await write_last_post_file(self.output_last[1], str(url))
         if not self.coomer_crawler:
             self.coomer_crawler = CoomerCrawler(include_id=self.include_id, scraping_mapper=self,
-                                                separate_posts=self.separate_posts)
+                                                separate_posts=self.separate_posts, quiet=self.quiet)
         domain_obj = await self.coomer_crawler.fetch(coomer_session, url)
         if title:
             await domain_obj.append_title(title)
@@ -135,7 +148,7 @@ class ScrapeMapper:
     async def Cyberfile(self, url: URL, title=None):
         cyberfile_session = Session(self.client)
         if not self.cyberfile_crawler:
-            self.cyberfile_crawler = CyberfileCrawler()
+            self.cyberfile_crawler = CyberfileCrawler(quiet=self.quiet)
         async with self.semaphore:
             domain_obj = await self.cyberfile_crawler.fetch(cyberfile_session, url)
         if title:
@@ -146,7 +159,7 @@ class ScrapeMapper:
     async def Erome(self, url: URL, title=None):
         erome_session = Session(self.client)
         if not self.erome_crawler:
-            self.erome_crawler = EromeCrawler(include_id=self.include_id)
+            self.erome_crawler = EromeCrawler(include_id=self.include_id, quiet=self.quiet)
         domain_obj = await self.erome_crawler.fetch(erome_session, url)
         if title:
             await domain_obj.append_title(title)
@@ -157,9 +170,9 @@ class ScrapeMapper:
         gofile_session = Session(self.client)
         if not self.gofile_crawler:
             try:
-                self.gofile_crawler = GofileCrawler()
+                self.gofile_crawler = GofileCrawler(quiet=self.quiet)
             except SystemExit:
-                await log("Couldn't start the GoFile crawler")
+                await log("Couldn't start the GoFile crawler", quiet=self.quiet)
                 return
         domain_obj = await self.gofile_crawler.fetch(gofile_session, url)
         if title:
@@ -170,18 +183,30 @@ class ScrapeMapper:
     async def HGameCG(self, url: URL, title=None):
         hgamecg_session = Session(self.client)
         if not self.hgamecg_crawler:
-            self.hgamecg_crawler = HGameCGCrawler()
+            self.hgamecg_crawler = HGameCGCrawler(quiet=self.quiet)
         domain_obj = await self.hgamecg_crawler.fetch(hgamecg_session, url)
         if title:
             await domain_obj.append_title(title)
         await self.Cascade.add_albums(domain_obj)
         await hgamecg_session.exit_handler()
 
+    async def ImgBox(self, url: URL, title=None):
+        imgbox_session = Session(self.client)
+        if not self.imgbox_crawler:
+            self.imgbox_crawler = ImgBoxCrawler(include_id=self.include_id, quiet=self.quiet)
+        domain_obj = await self.imgbox_crawler.fetch(imgbox_session, url)
+        if title:
+            await domain_obj.append_title(title)
+        await self.Cascade.add_albums(domain_obj)
+        await imgbox_session.exit_handler()
+
     async def Kemono(self, url: URL, title=None):
         kemono_session = Session(self.client)
+        if self.output_last[0]:
+            await write_last_post_file(self.output_last[1], str(url))
         if not self.kemono_crawler:
             self.kemono_crawler = KemonoCrawler(include_id=self.include_id, scraping_mapper=self,
-                                                separate_posts=self.separate_posts)
+                                                separate_posts=self.separate_posts, quiet=self.quiet)
         domain_obj = await self.kemono_crawler.fetch(kemono_session, url)
         if title:
             await domain_obj.append_title(title)
@@ -191,7 +216,7 @@ class ScrapeMapper:
     async def Gfycat(self, url: URL, title=None):
         gfycat_session = Session(self.client)
         if not self.gfycat_crawler:
-            self.gfycat_crawler = GfycatCrawler()
+            self.gfycat_crawler = GfycatCrawler(quiet=self.quiet)
         content_url = await self.gfycat_crawler.fetch(gfycat_session, url)
         if content_url:
             if title:
@@ -203,7 +228,7 @@ class ScrapeMapper:
     async def Pixeldrain(self, url: URL, title=None):
         pixeldrain_session = Session(self.client)
         if not self.pixeldrain_crawler:
-            self.pixeldrain_crawler = PixelDrainCrawler()
+            self.pixeldrain_crawler = PixelDrainCrawler(quiet=self.quiet)
         domain_obj = await self.pixeldrain_crawler.fetch(pixeldrain_session, url)
         if title:
             await domain_obj.append_title(title)
@@ -213,7 +238,7 @@ class ScrapeMapper:
     async def Postimg(self, url: URL, title=None):
         postimg_session = Session(self.client)
         if not self.postimg_crawler:
-            self.postimg_crawler = PostImgCrawler(include_id=self.include_id)
+            self.postimg_crawler = PostImgCrawler(include_id=self.include_id, quiet=self.quiet)
         domain_obj = await self.postimg_crawler.fetch(postimg_session, url)
         if title:
             await domain_obj.append_title(title)
@@ -223,7 +248,7 @@ class ScrapeMapper:
     async def Redgifs(self, url: URL, title=None):
         redgifs_session = Session(self.client)
         if not self.redgifs_crawler:
-            self.redgifs_crawler = RedGifsCrawler()
+            self.redgifs_crawler = RedGifsCrawler(quiet=self.quiet)
         content_url = await self.redgifs_crawler.fetch(redgifs_session, url)
         if content_url:
             if title:
@@ -235,7 +260,7 @@ class ScrapeMapper:
     async def Rule34(self, url: URL, title=None):
         rule34_session = Session(self.client)
         if not self.rule34_crawler:
-            self.rule34_crawler = Rule34Crawler()
+            self.rule34_crawler = Rule34Crawler(quiet=self.quiet)
         domain_obj = await self.rule34_crawler.fetch(rule34_session, url)
         
         if title:
@@ -247,7 +272,7 @@ class ScrapeMapper:
     async def Saint(self, url: URL, title=None):
         saint_session = Session(self.client)
         if not self.saint_crawler:
-            self.saint_crawler = SaintCrawler(include_id=self.include_id)
+            self.saint_crawler = SaintCrawler(include_id=self.include_id, quiet=self.quiet)
         domain_obj = await self.saint_crawler.fetch(saint_session, url)
         if title:
             await domain_obj.append_title(title)
@@ -257,7 +282,7 @@ class ScrapeMapper:
     async def ShareX(self, url: URL, title=None):
         sharex_session = Session(self.client)
         if not self.sharex_crawler:
-            self.sharex_crawler = ShareXCrawler(include_id=self.include_id)
+            self.sharex_crawler = ShareXCrawler(include_id=self.include_id, quiet=self.quiet)
         if "jpg.church" in url.host and sharex_session.client.ratelimit > 19:
             async with self.jpgchurch_semaphore:
                 async with self.jpgchurch_limiter:
@@ -277,7 +302,7 @@ class ScrapeMapper:
                                                                     auth=self.socialmediagirls_auth,
                                                                     scraping_mapper=self,
                                                                     separate_posts=self.separate_posts,
-                                                                    output_last=self.output_last)
+                                                                    output_last=self.output_last, quiet=self.quiet)
         async with self.forum_limiter:
             await self.Cascade.extend(await self.socialmediagirls_crawler.fetch(socialmediagirls_session, url))
         await socialmediagirls_session.exit_handler()
@@ -287,7 +312,7 @@ class ScrapeMapper:
         if not self.simpcity_crawler:
             self.simpcity_crawler = SimpCityCrawler(include_id=self.include_id, auth=self.simpcity_auth,
                                                     scraping_mapper=self, separate_posts=self.separate_posts,
-                                                    output_last=self.output_last)
+                                                    output_last=self.output_last, quiet=self.quiet)
         async with self.forum_limiter:
             await self.Cascade.extend(await self.simpcity_crawler.fetch(simpcity_session, url))
         await simpcity_session.exit_handler()
@@ -297,7 +322,7 @@ class ScrapeMapper:
         if not self.xbunker_crawler:
             self.xbunker_crawler = XBunkerCrawler(include_id=self.include_id, auth=self.xbunker_auth,
                                                   scraping_mapper=self, separate_posts=self.separate_posts,
-                                                  output_last=self.output_last)
+                                                  output_last=self.output_last, quiet=self.quiet)
         async with self.forum_limiter:
             await self.Cascade.extend(await self.xbunker_crawler.fetch(xbunker_session, url))
         await xbunker_session.exit_handler()
@@ -305,7 +330,7 @@ class ScrapeMapper:
     async def XBunkr(self, url: URL, title=None):
         xbunkr_session = Session(self.client)
         if not self.xbunkr_crawler:
-            self.xbunkr_crawler = XBunkrCrawler(include_id=self.include_id)
+            self.xbunkr_crawler = XBunkrCrawler(include_id=self.include_id, quiet=self.quiet)
         domain_obj = await self.xbunkr_crawler.fetch(xbunkr_session, url)
         if title:
             await domain_obj.append_title(title)
@@ -321,19 +346,24 @@ class ScrapeMapper:
             jd.connect(self.jdownloader_auth.username, self.jdownloader_auth.password)
             self.jdownloader_agent = jd.get_device(self.jdownloader_device)
         except:
-            await log("Failed jdownloader setup")
+            await log("Failed jdownloader setup", quiet=self.quiet)
             self.jdownloader_enable = False
 
+    async def close(self):
+        for crawler in self.crawlers:
+            crawler = None
+
+
     async def map_url(self, url_to_map: URL, title=None):
         if not url_to_map:
             return
         elif not url_to_map.host:
-            await log(str(url_to_map) + " is not supported currently.")
+            await log(str(url_to_map) + " is not supported currently.", quiet=self.quiet)
             return
         for key, value in self.mapping.items():
             if key in url_to_map.host:
                 if any(site in key for site in self.skip_data.sites):
-                    await log("Skipping scrape of " + str(url_to_map))
+                    await log("Skipping scrape of " + str(url_to_map), quiet=self.quiet)
                 else:
                     await value(url=url_to_map, title=title)
                 return
@@ -344,12 +374,17 @@ class ScrapeMapper:
             try:
                 if "facebook" in url_to_map.host.lower() or "instagram" in url_to_map.host.lower():
                     raise Exception("Blacklisted META")
-                await log("Sending " + str(url_to_map) + " to JDownloader")
-                self.jdownloader_agent.linkgrabber.add_links([{"autostart": False, "links": str(url_to_map)}])
+                await log("Sending " + str(url_to_map) + " to JDownloader", quiet=self.quiet)
+                self.jdownloader_agent.linkgrabber.add_links([{
+                    "autostart": False,
+                    "links": str(url_to_map),
+                    "packageName": title if title else "Cyberdrop-DL",
+                    "overwritePackagizerRules": True
+                    }])
             except Exception as e:
                 logging.debug(e)
-                await log("Failed to send " + str(url_to_map) + " to JDownloader")
+                await log("Failed to send " + str(url_to_map) + " to JDownloader", quiet=self.quiet)
         else:
-            await log(str(url_to_map) + " is not supported currently.")
+            await log("Not Supported: " + str(url_to_map), quiet=self.quiet)
             async with aiofiles.open("./Unsupported_Urls.txt", mode='a') as f:
                 await f.write(str(url_to_map)+"\n")
